@@ -1,6 +1,6 @@
 /* Generic BFD support for file formats.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1999, 2000, 2001, 2002,
-   2003, 2005, 2007 Free Software Foundation, Inc.
+   2003, 2005, 2007, 2008 Free Software Foundation, Inc.
    Written by Cygnus Support.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -125,6 +125,9 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
   int match_count;
   int ar_match_index;
 
+  if (matching != NULL)
+    *matching = NULL;
+
   if (!bfd_read_p (abfd)
       || (unsigned int) abfd->format >= (unsigned int) bfd_type_end)
     {
@@ -141,11 +144,10 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
   match_count = 0;
   ar_match_index = _bfd_target_vector_entries;
 
-  if (matching)
+  if (matching != NULL || *bfd_associated_vector != NULL)
     {
       bfd_size_type amt;
 
-      *matching = NULL;
       amt = sizeof (*matching_vector) * 2 * _bfd_target_vector_entries;
       matching_vector = bfd_malloc (amt);
       if (!matching_vector)
@@ -162,31 +164,12 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
   if (!abfd->target_defaulted)
     {
       if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0)	/* rewind! */
-	{
-	  if (matching)
-	    free (matching_vector);
-	  return FALSE;
-	}
+	goto err_ret;
 
       right_targ = BFD_SEND_FMT (abfd, _bfd_check_format, (abfd));
 
       if (right_targ)
-	{
-	  abfd->xvec = right_targ;	/* Set the target as returned.  */
-
-	  if (matching)
-	    free (matching_vector);
-
-	  /* If the file was opened for update, then `output_has_begun'
-	     some time ago when the file was created.  Do not recompute
-	     sections sizes or alignments in _bfd_set_section_contents.
-	     We can not set this flag until after checking the format,
-	     because it will interfere with creation of BFD sections.  */
-	  if (abfd->direction == both_direction)
-	    abfd->output_has_begun = TRUE;
-
-	  return TRUE;			/* File position has moved, BTW.  */
-	}
+	goto ok_ret;
 
       /* For a long time the code has dropped through to check all
 	 targets if the specified target was wrong.  I don't know why,
@@ -201,17 +184,7 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
 	 this sort, I changed this test to check only for the binary
 	 target.  */
       if (format == bfd_archive && save_targ == &binary_vec)
-	{
-	  abfd->xvec = save_targ;
-	  abfd->format = bfd_unknown;
-
-	  if (matching)
-	    free (matching_vector);
-
-	  bfd_set_error (bfd_error_file_not_recognized);
-
-	  return FALSE;
-	}
+	goto err_unrecog;
     }
 
   for (target = bfd_target_vector; *target != NULL; target++)
@@ -227,11 +200,7 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
       abfd->xvec = *target;	/* Change BFD's target temporarily.  */
 
       if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0)
-	{
-	  if (matching)
-	    free (matching_vector);
-	  return FALSE;
-	}
+	goto err_ret;
 
       /* If _bfd_check_format neglects to set bfd_error, assume
 	 bfd_error_wrong_format.  We didn't used to even pay any
@@ -241,7 +210,7 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
 
       temp = BFD_SEND_FMT (abfd, _bfd_check_format, (abfd));
 
-      if (temp)
+      if (temp && (abfd->format != bfd_archive || bfd_has_map (abfd)))
 	{
 	  /* This format checks out as ok!  */
 	  right_targ = temp;
@@ -255,33 +224,25 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
 	      break;
 	    }
 
-	  if (matching)
+	  if (matching_vector)
 	    matching_vector[match_count] = temp;
-
 	  match_count++;
 	}
-      else if ((err = bfd_get_error ()) == bfd_error_wrong_object_format
+      else if (temp
+	       || (err = bfd_get_error ()) == bfd_error_wrong_object_format
 	       || err == bfd_error_file_ambiguously_recognized)
 	{
-	  /* An archive with objects of the wrong type, or an
-	     ambiguous match.  We want this target to match if we get
-	     no better matches.  */
+	  /* An archive with no armap or objects of the wrong type,
+	     or an ambiguous match.  We want this target to match
+	     if we get no better matches.  */
 	  if (ar_right_targ != bfd_default_vector[0])
 	    ar_right_targ = *target;
-	  if (matching)
+	  if (matching_vector)
 	    matching_vector[ar_match_index] = *target;
 	  ar_match_index++;
 	}
       else if (err != bfd_error_wrong_format)
-	{
-	  abfd->xvec = save_targ;
-	  abfd->format = bfd_unknown;
-
-	  if (matching)
-	    free (matching_vector);
-
-	  return FALSE;
-	}
+	goto err_ret;
     }
 
   if (match_count == 0)
@@ -297,16 +258,14 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
 	{
 	  match_count = ar_match_index - _bfd_target_vector_entries;
 
-	  if (matching && match_count > 1)
+	  if (matching_vector && match_count > 1)
 	    memcpy (matching_vector,
 		    matching_vector + _bfd_target_vector_entries,
 		    sizeof (*matching_vector) * match_count);
 	}
     }
 
-  if (match_count > 1
-      && bfd_associated_vector != NULL
-      && matching)
+  if (match_count > 1)
     {
       const bfd_target * const *assoc = bfd_associated_vector;
 
@@ -328,11 +287,9 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
 
   if (match_count == 1)
     {
+    ok_ret:
       abfd->xvec = right_targ;		/* Change BFD's target permanently.  */
 
-      if (matching)
-	free (matching_vector);
-
       /* If the file was opened for update, then `output_has_begun'
 	 some time ago when the file was created.  Do not recompute
 	 sections sizes or alignments in _bfd_set_section_contents.
@@ -341,37 +298,39 @@ bfd_check_format_matches (bfd *abfd, bfd_format format, char ***matching)
       if (abfd->direction == both_direction)
 	abfd->output_has_begun = TRUE;
 
+      if (matching_vector)
+	free (matching_vector);
       return TRUE;			/* File position has moved, BTW.  */
     }
 
+  if (match_count == 0)
+    {
+    err_unrecog:
+      bfd_set_error (bfd_error_file_not_recognized);
+    err_ret:
+      abfd->xvec = save_targ;
+      abfd->format = bfd_unknown;
+      if (matching_vector)
+	free (matching_vector);
+      return FALSE;
+    }
+
   abfd->xvec = save_targ;		/* Restore original target type.  */
   abfd->format = bfd_unknown;		/* Restore original format.  */
+  bfd_set_error (bfd_error_file_ambiguously_recognized);
 
-  if (match_count == 0)
+  if (matching)
     {
-      bfd_set_error (bfd_error_file_not_recognized);
-
-      if (matching)
-	free (matching_vector);
-    }
-  else
-    {
-      bfd_set_error (bfd_error_file_ambiguously_recognized);
-
-      if (matching)
+      *matching = (char **) matching_vector;
+      matching_vector[match_count] = NULL;
+      /* Return target names.  This is a little nasty.  Maybe we
+	 should do another bfd_malloc?  */
+      while (--match_count >= 0)
 	{
-	  *matching = (char **) matching_vector;
-	  matching_vector[match_count] = NULL;
-	  /* Return target names.  This is a little nasty.  Maybe we
-	     should do another bfd_malloc?  */
-	  while (--match_count >= 0)
-	    {
-	      const char *name = matching_vector[match_count]->name;
-	      *(const char **) &matching_vector[match_count] = name;
-	    }
+	  const char *name = matching_vector[match_count]->name;
+	  *(const char **) &matching_vector[match_count] = name;
 	}
     }
-
   return FALSE;
 }
 
