@@ -2,34 +2,34 @@
    Copyright 1990, 1991, 1992, 1994, 1995, 2000, 2001, 2002, 2007
    Free Software Foundation, Inc.
 
-This file is part of BFD, the Binary File Descriptor library.
+   This file is part of BFD, the Binary File Descriptor library.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Written by Ian Dall
- *            19-Apr-94
- *
- * Formerly part of aout-pc532-mach.c. Split out to allow more
- * flexibility with multiple formats.
- *
- */
+              19-Apr-94
+
+   Formerly part of aout-pc532-mach.c. Split out to allow more
+   flexibility with multiple formats.  */
+
 /* This architecture has N_TXTOFF and N_TXTADDR defined as if
- * N_HEADER_IN_TEXT, but the a_text entry (text size) does not include the
- * space for the header. So we have N_HEADER_IN_TEXT defined to
- * 1 and specially define our own N_TXTSIZE
- */
+   N_HEADER_IN_TEXT, but the a_text entry (text size) does not include the
+   space for the header. So we have N_HEADER_IN_TEXT defined to
+   1 and specially define our own N_TXTSIZE.  */
 
 #define N_HEADER_IN_TEXT(x) 1
 #define N_TXTSIZE(x) ((x).a_text)
