@@ -1,5 +1,5 @@
 /* addr2line.c -- convert addresses to line number and function name
-   Copyright 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2006
+   Copyright 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007
    Free Software Foundation, Inc.
    Contributed by Ulrich Lauther <Ulrich.Lauther@mchp.siemens.de>
 
@@ -7,7 +7,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,7 +17,9 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
+   Foundation, 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
+
 
 /* Derived from objdump.c and nm.c by Ulrich.Lauther@mchp.siemens.de
 
@@ -29,15 +31,12 @@
    both forms write results to stdout, the second form reads addresses
    to be converted from stdin.  */
 
-#include "config.h"
-#include <string.h>
-
+#include "sysdep.h"
 #include "bfd.h"
 #include "getopt.h"
 #include "libiberty.h"
 #include "demangle.h"
 #include "bucomm.h"
-#include "budemang.h"
 
 static bfd_boolean unwind_inlines;	/* -i, unwind inlined functions. */
 static bfd_boolean with_functions;	/* -f, show function names.  */
@@ -49,14 +48,10 @@ static char **addr;		/* Hex addresses to process.  */
 
 static asymbol **syms;		/* Symbol table.  */
 
-/* 150 isn't special; it's just an arbitrary non-ASCII char value.  */
-#define OPTION_DEMANGLER	(150)
-
 static struct option long_options[] =
 {
   {"basenames", no_argument, NULL, 's'},
   {"demangle", optional_argument, NULL, 'C'},
-  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"exe", required_argument, NULL, 'e'},
   {"functions", no_argument, NULL, 'f'},
   {"inlines", no_argument, NULL, 'i'},
@@ -72,7 +67,6 @@ static void slurp_symtab (bfd *);
 static void find_address_in_section (bfd *, asection *, void *);
 static void find_offset_in_section (bfd *, asection *);
 static void translate_addresses (bfd *, asection *);
-static void process_file (const char *, const char *, const char *);
 
 /* Print a usage message to STREAM and exit with STATUS.  */
 
@@ -90,16 +84,13 @@ usage (FILE *stream, int status)
   -j --section=<name>    Read section-relative offsets instead of addresses\n\
   -s --basenames         Strip directory names\n\
   -f --functions         Show function names\n\
-  -C --demangle[=%s]\n\
-                         Demangle function names\n\
-     --demangler=<dso:function>\n\
-                         Set dso and demangler function\n\
+  -C --demangle[=style]  Demangle function names\n\
   -h --help              Display this information\n\
   -v --version           Display the program's version\n\
-\n"), get_demangler_list ());
+\n"));
 
   list_supported_targets (program_name, stream);
-  if (status == 0)
+  if (REPORT_BUGS_TO[0] && status == 0)
     fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
   exit (status);
 }
@@ -232,8 +223,9 @@ translate_addresses (bfd *abfd, asection *section)
 		  name = "??";
 		else if (do_demangle)
 		  {
-		    alloc = demangle (abfd, name);
-		    name = alloc;
+		    alloc = bfd_demangle (abfd, name, DMGL_ANSI | DMGL_PARAMS);
+		    if (alloc != NULL)
+		      name = alloc;
 		  }
 
 		printf ("%s\n", name);
@@ -268,9 +260,9 @@ translate_addresses (bfd *abfd, asection *section)
     }
 }
 
-/* Process a file.  */
+/* Process a file.  Returns an exit value for main().  */
 
-static void
+static int
 process_file (const char *file_name, const char *section_name,
 	      const char *target)
 {
@@ -279,7 +271,7 @@ process_file (const char *file_name, const char *section_name,
   char **matching;
 
   if (get_file_size (file_name) < 1)
-    return;
+    return 1;
 
   abfd = bfd_openr (file_name, target);
   if (abfd == NULL)
@@ -319,6 +311,8 @@ process_file (const char *file_name, const char *section_name,
     }
 
   bfd_close (abfd);
+
+  return 0;
 }
 
 int
@@ -328,7 +322,6 @@ main (int argc, char **argv)
   const char *section_name;
   char *target;
   int c;
-  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
   setlocale (LC_MESSAGES, "");
@@ -363,7 +356,16 @@ main (int argc, char **argv)
 	case 'C':
 	  do_demangle = TRUE;
 	  if (optarg != NULL)
-	    style = optarg;
+	    {
+	      enum demangling_styles style;
+
+	      style = cplus_demangle_name_to_style (optarg);
+	      if (style == unknown_demangling)
+		fatal (_("unknown demangling style `%s'"),
+		       optarg);
+
+	      cplus_demangle_set_style (style);
+	    }
 	  break;
 	case 'e':
 	  file_name = optarg;
@@ -388,24 +390,17 @@ main (int argc, char **argv)
 	case 'j':
 	  section_name = optarg;
 	  break;
-	case OPTION_DEMANGLER:
-	  demangler = optarg;
-	  break;
 	default:
 	  usage (stderr, 1);
 	  break;
 	}
     }
 
-  init_demangler (style, NULL, demangler);
-
   if (file_name == NULL)
     file_name = "a.out";
 
   addr = argv + optind;
   naddr = argc - optind;
 
-  process_file (file_name, section_name, target);
-
-  return 0;
+  return process_file (file_name, section_name, target);
 }
