@@ -6,7 +6,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
 #ifndef _BUCOMM_H
 #define _BUCOMM_H
@@ -26,6 +27,9 @@ const char *bfd_get_archive_filename (bfd *);
 
 void bfd_nonfatal (const char *);
 
+void bfd_nonfatal_message (const char *, const bfd *, const asection *,
+			   const char *, ...);
+
 void bfd_fatal (const char *) ATTRIBUTE_NORETURN;
 
 void report (const char *, va_list) ATTRIBUTE_PRINTF(1,0);
