@@ -1,12 +1,12 @@
 /* m68k-parse.h -- header file for m68k assembler
    Copyright 1987, 1991, 1992, 1993, 1994, 1995, 1996, 1999, 2000,
-   2003, 2004, 2005 Free Software Foundation, Inc.
+   2003, 2004, 2005, 2007 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -131,8 +131,8 @@ enum m68k_register
   MBAR,
   ASID,				/* m5475.  */
   CAC,  		        /* fido added this.  */
-  MBB,
-#define last_movec_reg MBB
+  MBO,
+#define last_movec_reg MBO
   /* End of movec ordering constraints.  */
 
   FPI,
