@@ -33,6 +33,7 @@
 #include "layout.h"
 #include "output.h"
 #include "symtab.h"
+#include "cref.h"
 #include "reloc.h"
 #include "object.h"
 #include "dynobj.h"
@@ -230,18 +231,6 @@ Object::handle_gnu_warning_section(const char* name, unsigned int shndx,
   return false;
 }
 
-// Class Relobj.
-
-// Return the output address of the input section SHNDX.
-uint64_t
-Relobj::output_section_address(unsigned int shndx) const
-{
-  section_offset_type offset;
-  Output_section* os = this->output_section(shndx, &offset);
-  gold_assert(os != NULL && offset != -1);
-  return os->address() + offset;
-}
-
 // Class Sized_relobj.
 
 template<int size, bool big_endian>
@@ -257,10 +246,13 @@ Sized_relobj<size, big_endian>::Sized_relobj(
     output_local_symbol_count_(0),
     output_local_dynsym_count_(0),
     symbols_(),
+    defined_count_(0),
     local_symbol_offset_(0),
     local_dynsym_offset_(0),
     local_values_(),
     local_got_offsets_(),
+    kept_comdat_sections_(),
+    comdat_groups_(),
     has_eh_frame_(false)
 {
 }
@@ -452,6 +444,12 @@ Sized_relobj<size, big_endian>::do_read_symbols(Read_symbols_data* sd)
   off_t readoff = this->has_eh_frame_ ? dataoff : extoff;
   section_size_type readsize = this->has_eh_frame_ ? datasize : extsize;
 
+  if (readsize == 0)
+    {
+      // No external symbols.  Also weird but also legal.
+      return;
+    }
+
   File_view* fvsymtab = this->get_lasting_view(readoff, readsize, true, false);
 
   // Read the section header for the symbol names.
@@ -604,7 +602,7 @@ Sized_relobj<size, big_endian>::include_section_group(
   bool include_group = ((flags & elfcpp::GRP_COMDAT) == 0
                         || layout->add_comdat(this, index, signature, true));
 
-  Relobj* kept_object = NULL;
+  Sized_relobj<size, big_endian>* kept_object = NULL;
   Comdat_group* kept_group = NULL;
 
   if (!include_group)
@@ -612,7 +610,9 @@ Sized_relobj<size, big_endian>::include_section_group(
       // This group is being discarded.  Find the object and group
       // that was kept in its place.
       unsigned int kept_group_index = 0;
-      kept_object = layout->find_kept_object(signature, &kept_group_index);
+      Relobj* kept_relobj = layout->find_kept_object(signature,
+                                                     &kept_group_index);
+      kept_object = static_cast<Sized_relobj<size, big_endian>*>(kept_relobj);
       if (kept_object != NULL)
         kept_group = kept_object->find_comdat_group(kept_group_index);
     }
@@ -743,9 +743,11 @@ Sized_relobj<size, big_endian>::include_linkonce_section(
       // In this case, the section index stored with the layout object
       // is the linkonce section that was kept.
       unsigned int kept_group_index = 0;
-      Relobj* kept_object = layout->find_kept_object(sig2, &kept_group_index);
-      if (kept_object != NULL)
+      Relobj* kept_relobj = layout->find_kept_object(sig2, &kept_group_index);
+      if (kept_relobj != NULL)
         {
+          Sized_relobj<size, big_endian>* kept_object
+              = static_cast<Sized_relobj<size, big_endian>*>(kept_relobj);
           Kept_comdat_section* kept =
             new Kept_comdat_section(kept_object, kept_group_index);
           this->set_kept_comdat_section(index, kept);
@@ -761,9 +763,11 @@ Sized_relobj<size, big_endian>::include_linkonce_section(
       // the group has only one member section.  Otherwise, it's not
       // worth the effort.
       unsigned int kept_group_index = 0;
-      Relobj* kept_object = layout->find_kept_object(sig1, &kept_group_index);
-      if (kept_object != NULL)
+      Relobj* kept_relobj = layout->find_kept_object(sig1, &kept_group_index);
+      if (kept_relobj != NULL)
         {
+          Sized_relobj<size, big_endian>* kept_object =
+              static_cast<Sized_relobj<size, big_endian>*>(kept_relobj);
           Comdat_group* kept_group =
             kept_object->find_comdat_group(kept_group_index);
           if (kept_group != NULL && kept_group->size() == 1)
@@ -835,8 +839,11 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 	}
     }
 
-  std::vector<Map_to_output>& map_sections(this->map_to_output());
-  map_sections.resize(shnum);
+  Output_sections& out_sections(this->output_sections());
+  std::vector<Address>& out_section_offsets(this->section_offsets_);
+
+  out_sections.resize(shnum);
+  out_section_offsets.resize(shnum);
 
   // If we are only linking for symbols, then there is nothing else to
   // do here.
@@ -918,7 +925,8 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
       if (discard)
 	{
 	  // Do not include this section in the link.
-	  map_sections[i].output_section = NULL;
+	  out_sections[i] = NULL;
+          out_section_offsets[i] = -1U;
 	  continue;
 	}
 
@@ -957,8 +965,11 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 					  reloc_shndx[i], reloc_type[i],
 					  &offset);
 
-      map_sections[i].output_section = os;
-      map_sections[i].offset = offset;
+      out_sections[i] = os;
+      if (offset == -1)
+        out_section_offsets[i] = -1U;
+      else
+        out_section_offsets[i] = convert_types<Address, off_t>(offset);
 
       // If this section requires special handling, and if there are
       // relocs that apply to it, then we must do the special handling
@@ -989,10 +1000,11 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 	  continue;
 	}
 
-      Output_section* data_section = map_sections[data_shndx].output_section;
+      Output_section* data_section = out_sections[data_shndx];
       if (data_section == NULL)
 	{
-	  map_sections[i].output_section = NULL;
+	  out_sections[i] = NULL;
+          out_section_offsets[i] = -1U;
 	  continue;
 	}
 
@@ -1001,8 +1013,8 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 
       Output_section* os = layout->layout_reloc(this, i, shdr, data_section,
 						rr);
-      map_sections[i].output_section = os;
-      map_sections[i].offset = -1;
+      out_sections[i] = os;
+      out_section_offsets[i] = -1U;
     }
 
   // Handle the .eh_frame sections at the end.
@@ -1028,8 +1040,11 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 						   reloc_shndx[i],
 						   reloc_type[i],
 						   &offset);
-      map_sections[i].output_section = os;
-      map_sections[i].offset = offset;
+      out_sections[i] = os;
+      if (offset == -1)
+        out_section_offsets[i] = -1U;
+      else
+        out_section_offsets[i] = convert_types<Address, off_t>(offset);
 
       // If this section requires special handling, and if there are
       // relocs that apply to it, then we must do the special handling
@@ -1074,7 +1089,8 @@ Sized_relobj<size, big_endian>::do_add_symbols(Symbol_table* symtab,
 			  sd->symbols->data() + sd->external_symbols_offset,
 			  symcount, this->local_symbol_count_,
 			  sym_names, sd->symbol_names_size,
-			  &this->symbols_);
+			  &this->symbols_,
+			  &this->defined_count_);
 
   delete sd->symbols;
   sd->symbols = NULL;
@@ -1125,7 +1141,7 @@ Sized_relobj<size, big_endian>::do_count_local_symbols(Stringpool* pool,
 
   // Loop over the local symbols.
 
-  const std::vector<Map_to_output>& mo(this->map_to_output());
+  const Output_sections& out_sections(this->output_sections());
   unsigned int shnum = this->shnum();
   unsigned int count = 0;
   unsigned int dyncount = 0;
@@ -1152,7 +1168,7 @@ Sized_relobj<size, big_endian>::do_count_local_symbols(Stringpool* pool,
 
       // Decide whether this symbol should go into the output file.
 
-      if (shndx < shnum && mo[shndx].output_section == NULL)
+      if (shndx < shnum && out_sections[shndx] == NULL)
         {
 	  lv.set_no_output_symtab_entry();
           gold_assert(!lv.needs_output_dynsym_entry());
@@ -1207,7 +1223,8 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
   const unsigned int loccount = this->local_symbol_count_;
   this->local_symbol_offset_ = off;
 
-  const std::vector<Map_to_output>& mo(this->map_to_output());
+  const Output_sections& out_sections(this->output_sections());
+  const std::vector<Address>& out_offsets(this->section_offsets_);
   unsigned int shnum = this->shnum();
 
   for (unsigned int i = 1; i < loccount; ++i)
@@ -1218,7 +1235,7 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
       unsigned int shndx = lv.input_shndx(&is_ordinary);
 
       // Set the output symbol value.
-      
+
       if (!is_ordinary)
 	{
 	  if (shndx == elfcpp::SHN_ABS || shndx == elfcpp::SHN_COMMON)
@@ -1239,7 +1256,7 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
 	      shndx = 0;
 	    }
 
-	  Output_section* os = mo[shndx].output_section;
+	  Output_section* os = out_sections[shndx];
 
 	  if (os == NULL)
 	    {
@@ -1249,7 +1266,7 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
               // so we leave the input value unchanged here.
 	      continue;
 	    }
-	  else if (mo[shndx].offset == -1)
+	  else if (out_offsets[shndx] == -1U)
 	    {
 	      // This is a SHF_MERGE section or one which otherwise
 	      // requires special handling.  We get the output address
@@ -1272,11 +1289,11 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
 	    }
           else if (lv.is_tls_symbol())
 	    lv.set_output_value(os->tls_offset()
-				+ mo[shndx].offset
+				+ out_offsets[shndx]
 				+ lv.input_value());
 	  else
 	    lv.set_output_value(os->address()
-				+ mo[shndx].offset
+				+ out_offsets[shndx]
 				+ lv.input_value());
 	}
 
@@ -1379,7 +1396,7 @@ Sized_relobj<size, big_endian>::write_local_symbols(
     dyn_oview = of->get_output_view(this->local_dynsym_offset_,
                                     dyn_output_size);
 
-  const std::vector<Map_to_output>& mo(this->map_to_output());
+  const Output_sections out_sections(this->output_sections());
 
   gold_assert(this->local_values_.size() == loccount);
 
@@ -1397,10 +1414,10 @@ Sized_relobj<size, big_endian>::write_local_symbols(
 						     &is_ordinary);
       if (is_ordinary)
 	{
-	  gold_assert(st_shndx < mo.size());
-	  if (mo[st_shndx].output_section == NULL)
+	  gold_assert(st_shndx < out_sections.size());
+	  if (out_sections[st_shndx] == NULL)
 	    continue;
-	  st_shndx = mo[st_shndx].output_section->out_shndx();
+	  st_shndx = out_sections[st_shndx]->out_shndx();
 	  if (st_shndx >= elfcpp::SHN_LORESERVE)
 	    {
 	      if (lv.needs_output_symtab_entry())
@@ -1554,13 +1571,37 @@ Sized_relobj<size, big_endian>::map_to_kept_section(
     {
       gold_assert(kept->object_ != NULL);
       *found = true;
-      return (static_cast<Address>
-              (kept->object_->output_section_address(kept->shndx_)));
+      Output_section* os = kept->object_->output_section(kept->shndx_);
+      Address offset = kept->object_->get_output_section_offset(kept->shndx_);
+      gold_assert(os != NULL && offset != -1U);
+      return os->address() + offset;
     }
   *found = false;
   return 0;
 }
 
+// Get symbol counts.
+
+template<int size, bool big_endian>
+void
+Sized_relobj<size, big_endian>::do_get_global_symbol_counts(
+    const Symbol_table*,
+    size_t* defined,
+    size_t* used) const
+{
+  *defined = this->defined_count_;
+  size_t count = 0;
+  for (Symbols::const_iterator p = this->symbols_.begin();
+       p != this->symbols_.end();
+       ++p)
+    if (*p != NULL
+	&& (*p)->source() == Symbol::FROM_OBJECT
+	&& (*p)->object() == this
+	&& (*p)->is_defined())
+      ++count;
+  *used = count;
+}
+
 // Input_objects methods.
 
 // Add a regular relocatable object to the list.  Return false if this
@@ -1615,6 +1656,14 @@ Input_objects::add_object(Object* obj)
 	}
     }
 
+  // Add this object to the cross-referencer if requested.
+  if (parameters->options().user_set_print_symbol_counts())
+    {
+      if (this->cref_ == NULL)
+	this->cref_ = new Cref();
+      this->cref_->add_object(obj);
+    }
+
   return true;
 }
 
@@ -1655,6 +1704,38 @@ Input_objects::check_dynamic_dependencies() const
     }
 }
 
+// Start processing an archive.
+
+void
+Input_objects::archive_start(Archive* archive)
+{
+  if (parameters->options().user_set_print_symbol_counts())
+    {
+      if (this->cref_ == NULL)
+	this->cref_ = new Cref();
+      this->cref_->add_archive_start(archive);
+    }
+}
+
+// Stop processing an archive.
+
+void
+Input_objects::archive_stop(Archive* archive)
+{
+  if (parameters->options().user_set_print_symbol_counts())
+    this->cref_->add_archive_stop(archive);
+}
+
+// Print symbol counts
+
+void
+Input_objects::print_symbol_counts(const Symbol_table* symtab) const
+{
+  if (parameters->options().user_set_print_symbol_counts()
+      && this->cref_ != NULL)
+    this->cref_->print_symbol_counts(symtab);
+}
+
 // Relocate_info methods.
 
 // Return a string describing the location of a relocation.  This is
