@@ -640,11 +640,13 @@ void
 General_options::finalize()
 {
   // Normalize the strip modifiers.  They have a total order:
-  // strip_all > strip_debug > strip_debug_gdb.  If one is true, set
-  // all beneath it to true as well.
+  // strip_all > strip_debug > strip_non_line > strip_debug_gdb.
+  // If one is true, set all beneath it to true as well.
   if (this->strip_all())
     this->set_strip_debug(true);
   if (this->strip_debug())
+    this->set_strip_debug_non_line(true);
+  if (this->strip_debug_non_line())
     this->set_strip_debug_gdb(true);
 
   // If the user specifies both -s and -r, convert the -s to -S.
@@ -697,6 +699,17 @@ General_options::finalize()
       this->set_do_demangle(getenv("COLLECT_NO_DEMANGLE") == NULL);
     }
 
+  // -M is equivalent to "-Map -".
+  if (this->print_map() && !this->user_set_Map())
+    {
+      this->set_Map("-");
+      this->set_user_set_Map();
+    }
+
+  // Using -n or -N implies -static.
+  if (this->nmagic() || this->omagic())
+    this->set_static(true);
+
   // If --thread_count is specified, it applies to
   // --thread-count-{initial,middle,final}, though it doesn't override
   // them.
@@ -751,6 +764,9 @@ General_options::finalize()
       this->add_to_library_path_with_sysroot("/usr/lib");
     }
 
+  if (this->shared() && !this->user_set_allow_shlib_undefined())
+    this->set_allow_shlib_undefined(true);
+
   // Normalize library_path() by adding the sysroot to all directories
   // in the path, as appropriate.
   this->add_sysroot();
