@@ -52,22 +52,31 @@ START_RELOC_NUMBERS (elf_m68k_reloc_type)
   RELOC_NUMBER (R_68K_GNU_VTENTRY, 24)
 END_RELOC_NUMBERS (R_68K_max)
 
+/* We use the top 24 bits to encode information about the
+   architecture variant.  */
 #define EF_M68K_CPU32    0x00810000
 #define EF_M68K_M68000   0x01000000
 #define EF_M68K_CFV4E    0x00008000
+#define EF_M68K_FIDO     0x02000000
+#define EF_M68K_ARCH_MASK						\
+  (EF_M68K_M68000 | EF_M68K_CPU32 | EF_M68K_CFV4E | EF_M68K_FIDO)
 
 /* We use the bottom 8 bits to encode information about the
-   coldfire variant.  */
-#define EF_M68K_ISA_MASK	0x07  /* Which ISA */
-#define EF_M68K_ISA_A		0x01
-#define EF_M68K_ISA_A_PLUS	0x02
-#define EF_M68K_ISA_B		0x03
-#define EF_M68K_HW_DIV		0x08  /* Has HW divide */
-#define EF_M68K_MAC_MASK	0x30 
-#define EF_M68K_MAC		0x10  /* Has MAC */
-#define EF_M68K_EMAC		0x20  /* Has EMAC */
-#define EF_M68K_USP		0x40  /* Has USP insns */
-#define EF_M68K_FLOAT		0x80  /* Has float insns */
+   coldfire variant.  If we use any of these bits, the top 24 bits are
+   either 0 or EF_M68K_CFV4E.  */
+#define EF_M68K_CF_ISA_MASK	0x0F  /* Which ISA */
+#define EF_M68K_CF_ISA_A_NODIV	0x01  /* ISA A except for div */
+#define EF_M68K_CF_ISA_A	0x02
+#define EF_M68K_CF_ISA_A_PLUS	0x03
+#define EF_M68K_CF_ISA_B_NOUSP	0x04  /* ISA_B except for USP */
+#define EF_M68K_CF_ISA_B	0x05
+#define EF_M68K_CF_ISA_C	0x06
+#define EF_M68K_CF_ISA_C_NODIV	0x07  /* ISA C except for div */
+#define EF_M68K_CF_MAC_MASK	0x30 
+#define EF_M68K_CF_MAC		0x10  /* MAC */
+#define EF_M68K_CF_EMAC		0x20  /* EMAC */
+#define EF_M68K_CF_EMAC_B	0x30  /* EMAC_B */
+#define EF_M68K_CF_FLOAT	0x40  /* Has float insns */
 #define EF_M68K_CF_MASK		0xFF
      
 #endif
