@@ -1,5 +1,7 @@
 MACHINE=
 SCRIPT_NAME=elf
+TEMPLATE_NAME=generic
+EXTRA_EM_FILE=genelf
 OUTPUT_FORMAT="elf32-fr30"
 TEXT_START_ADDR=0x10000
 ARCH=fr30
@@ -7,4 +9,4 @@ MAXPAGESIZE=256
 ENTRY=_start
 EMBEDDED=yes
 NOP=0x9fa09fa0
-OTHER_END_SYMBOLS='PROVIDE (__stack = 0x200000);'
+OTHER_SYMBOLS='PROVIDE (__stack = 0x200000);'
