@@ -1,8 +1,10 @@
 SCRIPT_NAME=elf
 OUTPUT_FORMAT="elf32-vax"
 TEXT_START_ADDR=0x10000
-MAXPAGESIZE=0x10000
-NONPAGED_TEXT_START_ADDR=0x1000
+case ${LD_FLAG} in
+    n|N)	TEXT_START_ADDR=0x1000 ;;
+esac
+MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
 ARCH=vax
 MACHINE=
 NOP=0x0101
