@@ -14,12 +14,12 @@ TINY_READONLY_SECTION=".tinyrodata :
   {
 	*(.tinyrodata)
   } =0"
-TINY_DATA_SECTION=".tinydata	0xff8000 :
+TINY_DATA_SECTION=".tinydata	${RELOCATING+0xff8000} :
   {
 	*(.tinydata)
-	_tinydata = .;
+        ${RELOCATING+ _tinydata = .; }
   }"
-TINY_BSS_SECTION=".tinybss	: AT (_tinydata)
+TINY_BSS_SECTION=".tinybss	: ${RELOCATING+AT (_tinydata)}
   {
 	*(.tinybss)
   }"
