@@ -1,5 +1,9 @@
-MACHINE=
+# If you change this file, please also look at files which source this one:
+# mn10300.sh
+
 SCRIPT_NAME=elf
+TEMPLATE_NAME=generic
+EXTRA_EM_FILE=genelf
 OUTPUT_FORMAT="elf32-mn10200"
 TEXT_START_ADDR=0x0
 ARCH=mn10200
@@ -8,9 +12,8 @@ MAXPAGESIZE=1
 ENTRY=_start
 EMBEDDED=yes
 
-# Hmmm, there's got to be a better way.  This sets the stack to the
-# top of the simulator memory (2^19 bytes).
-OTHER_RELOCATING_SECTIONS='.stack 0x80000 : { _stack = .; *(.stack) }'
+# This sets the stack to the top of the simulator memory (2^19 bytes).
+STACK_ADDR=0x80000
 
 # These are for compatibility with the COFF toolchain.
 # XXX These should definitely disappear.
