@@ -3,10 +3,9 @@
 SCRIPT_NAME=elf
 OUTPUT_FORMAT=${OUTPUT_FORMAT-"elf32-sh64"}
 TEXT_START_ADDR=0x1000
-STACK_ADDR='ALIGN (0x40000) + 0x40000'
 MAXPAGESIZE=128
 ARCH=sh
-MACHINE=
+MACHINE=sh5
 ALIGNMENT=8
 TEMPLATE_NAME=elf32
 GENERATE_SHLIB_SCRIPT=yes
@@ -34,9 +33,25 @@ DTOR_END='___dtors_end = .;'
 # Do not use the varname=${varname-'string'} construct here; there are
 # problems with that on some shells (e.g. on Solaris) where there is a bug
 # that trigs when $varname contains a "}".
-OTHER_SECTIONS='
- .cranges 0 : { *(.cranges) }
-'
+# The effect of the .stack definition is like setting STACK_ADDR to 0x80000,
+# except that the setting can be overridden, e.g. --defsym _stack=0xff000,
+# and that we put an extra sentinal value at the bottom.
+# N.B. We can't use PROVIDE to set the default value in a symbol because
+# the address is needed to place the .stack section, which in turn is needed
+# to hold the sentinel value(s).
+test -z "$CREATE_SHLIB" && OTHER_SECTIONS="
+  .stack ${RELOCATING-0}${RELOCATING+(DEFINED(_stack) ? _stack : ALIGN (0x40000) + 0x80000)} :
+  {
+    ${RELOCATING+_stack = .;}
+    *(.stack)
+    LONG(0xdeaddead)
+  }
+  .cranges 0 : { *(.cranges) }
+"
+# We do not need .stack for shared library.
+test -n "$CREATE_SHLIB" && OTHER_SECTIONS="
+  .cranges 0 : { *(.cranges) }
+"
 
 # We need to adjust sizes in the .cranges section after relaxation, so
 # we need an after_allocation function, and it goes in this file.
