@@ -22,6 +22,24 @@ FINI_START='_fini = .;
 FINI_END="KEEP (*(.fini\$99));
           PROVIDE (${SYMPREFIX}_etext = .);"
 
+OTHER_READONLY_SECTIONS=".tls_data ${RELOCATING-0} : {${RELOCATING+
+    __wrs_rtp_tls_data_start = .;
+    ___wrs_rtp_tls_data_start = .;}
+    *(.tls_data${RELOCATING+ .tls_data.*})
+  }${RELOCATING+
+  __wrs_rtp_tls_data_size = . - __wrs_rtp_tls_data_start;
+  ___wrs_rtp_tls_data_size = . - __wrs_rtp_tls_data_start;
+  __wrs_rtp_tls_data_align = ALIGNOF(.tls_data);
+  ___wrs_rtp_tls_data_align = ALIGNOF(.tls_data);}"
+
+OTHER_READWRITE_SECTIONS=".tls_vars ${RELOCATING-0} : {${RELOCATING+
+    __wrs_rtp_tls_vars_start = .;
+    ___wrs_rtp_tls_vars_start = .;}
+    *(.tls_vars${RELOCATING+ .tls_vars.*})
+  }${RELOCATING+
+  __wrs_rtp_tls_vars_size = SIZEOF(.tls_vars);
+  ___wrs_rtp_tls_vars_size = SIZEOF(.tls_vars);}"
+
 ETEXT_NAME=etext_unrelocated
 OTHER_END_SYMBOLS="PROVIDE (${SYMPREFIX}_ehdr = ${TEXT_START_ADDR});"
 DATA_END_SYMBOLS=".edata : { PROVIDE (${SYMPREFIX}_edata = .); }"
