@@ -114,8 +114,6 @@ static const char *get_sysroot
   (int, char **);
 static char *get_emulation
   (int, char **);
-static void set_scripts_dir
-  (void);
 static bfd_boolean add_archive_element
   (struct bfd_link_info *, bfd *, const char *);
 static bfd_boolean multiple_definition
@@ -276,6 +274,7 @@ main (int argc, char **argv)
   link_info.relax_pass = 1;
   link_info.pei386_auto_import = -1;
   link_info.spare_dynamic_tags = 5;
+  link_info.path_separator = ':';
 
   ldfile_add_arch ("");
   emulation = get_emulation (argc, argv);
@@ -357,10 +356,6 @@ main (int argc, char **argv)
 	link_info.discard = discard_all;
     }
 
-  /* This essentially adds another -L directory so this must be done after
-     the -L's in argv have been processed.  */
-  set_scripts_dir ();
-
   /* If we have not already opened and parsed a linker script,
      try the default script from command line first.  */
   if (saved_script_handle == NULL
@@ -670,106 +665,6 @@ get_emulation (int argc, char **argv)
   return emulation;
 }
 
-/* If directory DIR contains an "ldscripts" subdirectory,
-   add DIR to the library search path and return TRUE,
-   else return FALSE.  */
-
-static bfd_boolean
-check_for_scripts_dir (char *dir)
-{
-  size_t dirlen;
-  char *buf;
-  struct stat s;
-  bfd_boolean res;
-
-  dirlen = strlen (dir);
-  /* sizeof counts the terminating NUL.  */
-  buf = xmalloc (dirlen + sizeof ("/ldscripts"));
-  sprintf (buf, "%s/ldscripts", dir);
-
-  res = stat (buf, &s) == 0 && S_ISDIR (s.st_mode);
-  free (buf);
-  if (res)
-    ldfile_add_library_path (dir, FALSE);
-  return res;
-}
-
-/* Set the default directory for finding script files.
-   Libraries will be searched for here too, but that's ok.
-   We look for the "ldscripts" directory in:
-
-   SCRIPTDIR (passed from Makefile)
-	     (adjusted according to the current location of the binary)
-   SCRIPTDIR (passed from Makefile)
-   the dir where this program is (for using it from the build tree)
-   the dir where this program is/../lib
-	     (for installing the tool suite elsewhere).  */
-
-static void
-set_scripts_dir (void)
-{
-  char *end, *dir;
-  size_t dirlen;
-  bfd_boolean found;
-
-  dir = make_relative_prefix (program_name, BINDIR, SCRIPTDIR);
-  if (dir)
-    {
-      found = check_for_scripts_dir (dir);
-      free (dir);
-      if (found)
-	return;
-    }
-
-  dir = make_relative_prefix (program_name, TOOLBINDIR, SCRIPTDIR);
-  if (dir)
-    {
-      found = check_for_scripts_dir (dir);
-      free (dir);
-      if (found)
-	return;
-    }
-
-  if (check_for_scripts_dir (SCRIPTDIR))
-    /* We've been installed normally.  */
-    return;
-
-  /* Look for "ldscripts" in the dir where our binary is.  */
-  end = strrchr (program_name, '/');
-#ifdef HAVE_DOS_BASED_FILE_SYSTEM
-  {
-    /* We could have \foo\bar, or /foo\bar.  */
-    char *bslash = strrchr (program_name, '\\');
-
-    if (end == NULL || (bslash != NULL && bslash > end))
-      end = bslash;
-  }
-#endif
-
-  if (end == NULL)
-    /* Don't look for ldscripts in the current directory.  There is
-       too much potential for confusion.  */
-    return;
-
-  dirlen = end - program_name;
-  /* Make a copy of program_name in dir.
-     Leave room for later "/../lib".  */
-  dir = xmalloc (dirlen + 8);
-  strncpy (dir, program_name, dirlen);
-  dir[dirlen] = '\0';
-
-  if (check_for_scripts_dir (dir))
-    {
-      free (dir);
-      return;
-    }
-
-  /* Look for "ldscripts" in <the dir where our binary is>/../lib.  */
-  strcpy (dir + dirlen, "/../lib");
-  check_for_scripts_dir (dir);
-  free (dir);
-}
-
 void
 add_ysym (const char *name)
 {
@@ -883,21 +778,10 @@ add_archive_element (struct bfd_link_info *info,
 {
   lang_input_statement_type *input;
 
-  input = xmalloc (sizeof (lang_input_statement_type));
+  input = xcalloc (1, sizeof (lang_input_statement_type));
   input->filename = abfd->filename;
   input->local_sym_name = abfd->filename;
   input->the_bfd = abfd;
-  input->asymbols = NULL;
-  input->next = NULL;
-  input->just_syms_flag = FALSE;
-  input->loaded = FALSE;
-  input->search_dirs_flag = FALSE;
-
-  /* FIXME: The following fields are not set: header.next,
-     header.type, closed, passive_position, symbol_count,
-     next_real_file, is_archive, target, real.  This bit of code is
-     from the old decode_library_subfile function.  I don't know
-     whether any of those fields matters.  */
 
   ldlang_add_file (input);
 
@@ -1212,45 +1096,22 @@ warning_callback (struct bfd_link_info *info ATTRIBUTE_UNUSED,
     einfo ("%B: %s%s\n", abfd, _("warning: "), warning);
   else
     {
-      lang_input_statement_type *entry;
-      asymbol **asymbols;
       struct warning_callback_info info;
 
       /* Look through the relocs to see if we can find a plausible
 	 address.  */
-      entry = (lang_input_statement_type *) abfd->usrdata;
-      if (entry != NULL && entry->asymbols != NULL)
-	asymbols = entry->asymbols;
-      else
-	{
-	  long symsize;
-	  long symbol_count;
 
-	  symsize = bfd_get_symtab_upper_bound (abfd);
-	  if (symsize < 0)
-	    einfo (_("%B%F: could not read symbols: %E\n"), abfd);
-	  asymbols = xmalloc (symsize);
-	  symbol_count = bfd_canonicalize_symtab (abfd, asymbols);
-	  if (symbol_count < 0)
-	    einfo (_("%B%F: could not read symbols: %E\n"), abfd);
-	  if (entry != NULL)
-	    {
-	      entry->asymbols = asymbols;
-	      entry->symbol_count = symbol_count;
-	    }
-	}
+      if (!bfd_generic_link_read_symbols (abfd))
+	einfo (_("%B%F: could not read symbols: %E\n"), abfd);
 
       info.found = FALSE;
       info.warning = warning;
       info.symbol = symbol;
-      info.asymbols = asymbols;
+      info.asymbols = bfd_get_outsymbols (abfd);
       bfd_map_over_sections (abfd, warning_find_reloc, &info);
 
       if (! info.found)
 	einfo ("%B: %s%s\n", abfd, _("warning: "), warning);
-
-      if (entry == NULL)
-	free (asymbols);
     }
 
   return TRUE;
