@@ -5,32 +5,32 @@
 #readelf: -WSsrl
 #target: powerpc64*-*-*
 
-There are 17 section headers.*
+There are [0-9]+ section headers, starting at offset 0x[0-9a-f]+:
 
 Section Headers:
  +\[Nr\] Name +Type +Address +Off +Size +ES Flg Lk Inf Al
- +\[ 0\] +NULL +0+ 0+ 0+ 0+ +0 +0 +0
- +\[ 1\] \.interp +.*
- +\[ 2\] \.hash +.*
- +\[ 3\] \.dynsym +.*
- +\[ 4\] \.dynstr +.*
- +\[ 5\] \.rela\.dyn +.*
- +\[ 6\] \.rela\.plt +.*
- +\[ 7\] \.text +PROGBITS .* 0+b8 0+ +AX +0 +0 +8
- +\[ 8\] \.tdata +PROGBITS .* 0+38 0+ WAT +0 +0 +8
- +\[ 9\] \.tbss +NOBITS .* 0+38 0+ WAT +0 +0 +8
- +\[10\] \.dynamic +DYNAMIC .* 0+150 10 +WA +4 +0 +8
- +\[11\] \.branch_lt +PROGBITS .* 0+ 0+ +WA +0 +0 +8
- +\[12\] \.got +PROGBITS .* 0+58 08 +WA +0 +0 +8
- +\[13\] \.plt +.*
- +\[14\] \.shstrtab +.*
- +\[15\] \.symtab +.*
- +\[16\] \.strtab +.*
+ +\[[ 0-9]+\] +NULL +0+ 0+ 0+ 0+ +0 +0 +0
+ +\[[ 0-9]+\] \.interp +.*
+ +\[[ 0-9]+\] \.hash +.*
+ +\[[ 0-9]+\] \.dynsym +.*
+ +\[[ 0-9]+\] \.dynstr +.*
+ +\[[ 0-9]+\] \.rela\.dyn +.*
+ +\[[ 0-9]+\] \.rela\.plt +.*
+ +\[[ 0-9]+\] \.text +PROGBITS .* 0+b8 0+ +AX +0 +0 +8
+ +\[[ 0-9]+\] \.tdata +PROGBITS .* 0+38 0+ WAT +0 +0 +8
+ +\[[ 0-9]+\] \.tbss +NOBITS .* 0+38 0+ WAT +0 +0 +8
+ +\[[ 0-9]+\] \.dynamic +DYNAMIC .* 0+150 10 +WA +4 +0 +8
+ +\[[ 0-9]+\] \.branch_lt +PROGBITS .* 0+ 0+ +WA +0 +0 +8
+ +\[[ 0-9]+\] \.got +PROGBITS .* 0+58 08 +WA +0 +0 +8
+ +\[[ 0-9]+\] \.plt +.*
+ +\[[ 0-9]+\] \.shstrtab +.*
+ +\[[ 0-9]+\] \.symtab +.*
+ +\[[ 0-9]+\] \.strtab +.*
 #...
 
 Elf file type is EXEC \(Executable file\)
 Entry point .*
-There are 6 program headers.*
+There are [0-9]+ program headers, starting at offset [0-9]+
 
 Program Headers:
  +Type +Offset +VirtAddr +PhysAddr +FileSiz +MemSiz +Flg Align
@@ -71,7 +71,7 @@ Symbol table '\.dynsym' contains [0-9]+ entries:
 .* NOTYPE +GLOBAL DEFAULT +ABS _edata
 .* NOTYPE +GLOBAL DEFAULT +ABS _end
 
-Symbol table '\.symtab' contains .* entries:
+Symbol table '\.symtab' contains [0-9]+ entries:
  +Num: +Value +Size Type +Bind +Vis +Ndx Name
 .* NOTYPE +LOCAL +DEFAULT +UND 
 .* SECTION LOCAL +DEFAULT +1 
